// libTorrent - BitTorrent library
// Copyright (C) 2005, Jari Sundell
//
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
// 
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
// 
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
//
// Contact:  Jari Sundell <jaris@ifi.uio.no>
//
//           Skomakerveien 33
//           3185 Skoppum, NORWAY

#ifndef LIBTORRENT_PATH_H
#define LIBTORRENT_PATH_H

#include <string>
#include <list>

namespace torrent {

// Tip of the century, use a blank first path to get root and "." to get current dir.

class Path {
public:
  typedef std::list<std::string> List;

  Path() {}
  Path(const std::string& p, bool literal = false);
  Path(const List& l) : m_list(l) {}

  List&              list() { return m_list; }

  std::string        path(bool escaped = true);

  static std::string escape(const std::string& s);

  static void        mkdir(const std::string& root,
			   List::const_iterator pathBegin, List::const_iterator pathEnd,
			   List::const_iterator ignoreBegin, List::const_iterator ignoreEnd,
			   unsigned int umask = 0777);
  
  static void        mkdir(const std::string& dir, unsigned int umask = 0777);

private:
  List m_list;
};

}

#endif

